//
// Filter.cpp
//
// Sample Filter Code to work with Hethmon Brothers
// Inet.Mail Perseus Version 2
//

//
// Copyright 2002 Hethmon Software. All rights reserved.
//
// This sample code may be used to create filters for use
// with Inet.Mail Perseus from Hethmon Software.
//


#ifdef __OS2__
  #define INCL_DOS
  #define OS2
  #include <os2.h>
#elif _WIN32
  #include <winsock2.h>
  #include <windows.h>
  #ifndef FALSE
     #define FALSE 0
  #endif
  #ifndef TRUE
     #define TRUE  1
  #endif
#endif

#include <types.h>
#include <string.h>
#include "filter.h"

// ------------------------------------------------------------------
//
// Your dll must define 2 entry points to work. The first is called
// when your dll is first loaded and the second is called for each
// event.
//
// The main entry point must be declared as:
//
//   u_long imMain(int iType, 
//                 u_long * ulEvents, 
//                 u_long * ulPriority, 
//                 char ** szDomain)
//
//   iType is an integer value declared as either INETMAIL_STARTUP
//   or INETMAIL_STOP. INETMAIL_STARTUP is passed when your dll is
//   loaded. INETMAIL_STOP is passed when your dll is unloaded.
//
//   ulEvents is an u_long value which your filter sets to determine
//   what events it will receive. It can be set to be a combination
//   of INETMAIL_HELO, INETMAIL_MAIL, INETMAIL_RCPT, and INETMAIL_DATA.
//
//   ulPriority is an u_long value which your filter sets to determine
//   in which order it is called. Zero (0) is the highest priority. Filters
//   with equal priorities will be called in a random manner. All filters
//   are called until one returns an event handled code.
//
//   szDomain is a character string which your filter can set to only
//   receive events for that domain. If left null, your filter receives
//   events for all domains.
//
//   A return code of zero (0) is considered success. Any other value
//   returned will result in the filter being unloaded.
//
//
// The event entry point must be declared as:
//
//   u_long imEvent(u_long ulEvent, 
//                  char * szDomain, 
//                  void * pData,
//                  char ** szMsg)
//
//   ulEvent is the current event type: INETMAIL_HELO, INETMAIL_MAIL, 
//   INETMAIL_RCPT, or INETMAIL_DATA.
//
//   szDomain is the domain name the current event is happening on.
//
//   pData is a pointer to the data structure your filter can inspect
//   and manipulate. For the INETMAIL_HELO, a FilterHelo structure is passed.
//   For INETMAIL_DATA events, a FilterMessage class object is passed. For 
//   INETMAIL_MAIL and INETMAIL_RCPT, a FilterRecipient class object is passed.
//
//   szMsg is a return parameter for your filter to provide a text
//   message giving the reason why the request was denied. It may not
//   exceed 950 bytes in size. You must allocate the memory for szMsg.
//   Inet.Mail will free the memory using the delete operator. Inet.Mail
//   only deletes the memory if the return code is INETMAIL_RC_DENY_REQUEST.
//
//   3 return codes are defined: INETMAIL_RC_DENY_REQUEST, INETMAIL_RC_NEXT_FILTER
//   and INETMAIL_RC_EVENT_HANDLED. INETMAIL_RC_DENY_REQUEST will cause
//   the request to return an error code. You may optionally set a return
//   message in the szMsg parameter not to exceed 950 bytes in size.
//   INETMAIL_RC_NEXT_FILTER means to call any other filters for this event.
//   INETMAIL_RC_EVENT_HANDLED means to cease any further filter processing.
//

// ***** IMPORTANT *****
//
// Your code must be compiled as C++. The data object passed to the
// imEvent function is a C++ class object. In your DEF file, you must
// then alias your mangled functions to the standard names. The sample
// DEF file does just this. Mangled names can be pulled from the object
// file using the CPPFILT utility with IBM Visual Age C++ 3.

// ------------------------------------------------------------------

u_long _Export imMain(int iType, 
                      u_long * ulEvents, 
                      u_long * ulPriority, 
                      char ** szDomain) {
  if (iType == INETMAIL_STARTUP) {
    *ulEvents = INETMAIL_HELO | INETMAIL_MAIL | INETMAIL_RCPT | INETMAIL_DATA;
    *ulPriority = 0;
    }
  return 0;
}

// ------------------------------------------------------------------

u_long _Export imEvent(u_long ulEvent, 
                       char * szDomain, 
                       void * pData,
                       char ** szMsg) {

  FilterHelo * fhHelo;
  FilterRecipient * frRcpt;
  FilterMessage * fmMsg;

  if (ulEvent == INETMAIL_HELO) {
    fhHelo = (FilterHelo *) pData;
    // Check and see where the message is coming from.
    if (strcmp(fhHelo->szHeloName, "uce.example.com") == 0) {
      *szMsg = new char[256];
      strcpy(*szMsg, "Mail is not accepted from UCE hosts.");
      return INETMAIL_RC_DENY_REQUEST;
      }
    // This one is ok. Let the next filter look at it.
    return INETMAIL_RC_NEXT_FILTER;
    }
  else if (ulEvent == INETMAIL_MAIL) {
    frRcpt = (FilterRecipient *) pData;
    // Don't accept mail from people who send uce
    if (strcmp(frRcpt->szRcpt, "uce@example.com") == 0) {
      *szMsg = new char[256];
      strcpy(*szMsg, "Mail is not accepted from UCE people.");
      return INETMAIL_RC_DENY_REQUEST;
      }
    return INETMAIL_RC_NEXT_FILTER;
    }
  else if (ulEvent == INETMAIL_RCPT) {
    return INETMAIL_RC_NEXT_FILTER;
//    *szMsg = new char[256];
//    strcpy(*szMsg, "That recipient is not allowed");
//    return INETMAIL_RC_DENY_REQUEST;
    }
  else if (ulEvent == INETMAIL_DATA) {
    return INETMAIL_RC_NEXT_FILTER;
//    *szMsg = new char[256];
//    strcpy(*szMsg, "No spam allowed here");
//    return INETMAIL_RC_DENY_REQUEST;
    }
  return INETMAIL_RC_NEXT_FILTER;
}

// ------------------------------------------------------------------
// ------------------------------------------------------------------
// ------------------------------------------------------------------