//
// Filter.h
//
// Sample Filter Code to work with Hethmon Brothers
// Inet.Mail Perseus Version 2
//

//
// Copyright 2002 Hethmon Software. All rights reserved.
//
// This sample code may be used to create filters for use
// with Inet.Mail Perseus from Hethmon Software.
//

//
// Filter Class File
//

#ifndef _HETHMON_FILTER_H_
#define _HETHMON_FILTER_H_

// imMain Event Types
#define INETMAIL_STARTUP    1
#define INETMAIL_STOP       2

// imEvent Event Types
#define INETMAIL_HELO   0x00000001  // ---- ---- ---- ---- ---- ---- ---- ---1
#define INETMAIL_MAIL   0x00000002  // ---- ---- ---- ---- ---- ---- ---- --1-
#define INETMAIL_RCPT   0x00000004  // ---- ---- ---- ---- ---- ---- ---- -1--
#define INETMAIL_DATA   0x00000008  // ---- ---- ---- ---- ---- ---- ---- 1---

// imEvent Return Codes
#define INETMAIL_RC_DENY_REQUEST    1
#define INETMAIL_RC_NEXT_FILTER     2
#define INETMAIL_RC_EVENT_HANDLED   3

// Defines for different user types possible.
// These are the possible values for the iUserType property
// of the FilterRecipient class.
#define U_LOCAL     0  // a local user
#define U_WILLRELAY 1  // a user we will relay for
#define U_WONTRELAY 2  // a user we will not relay for
#define U_UNKNOWN   3  // unknown
#define U_GATE      4  //

// These are the possible values for the iAddr property of
// the FilterRecipient class.
#define NO_ADDR     0  // emtpy <>
#define ONE_ADDR    1  // exactly one address found
#define NULL_ADDR   2  // no <> found at all
#define BAD_ADDR    3  // malformed address found

// ----------------------------------------------------------------------------

class FilterHelo {
  public:

  char * szHeloName;
  char * szIpAddress;
  char * szIpName;
};

// ----------------------------------------------------------------------------

class FilterRecipient {
  public:

  char * szRcpt,     // The full user@domain.
       * szUser,     // The user only.
       * szDomain;   // The domain only.
  int iAddr,         // Address type determined
      iUserType;     // The type of user.
  FilterRecipient * rNext, // Link to next recipient.
                  * rPrev; // Link to next send recipient.

  FilterRecipient() {
    szRcpt = NULL;
    szUser = NULL;
    szDomain = NULL;
    iAddr = NULL_ADDR;
    iUserType = U_UNKNOWN;
    rNext = NULL;
    rPrev = NULL;
    };
  ~FilterRecipient() {
    if (szRcpt) delete [] szRcpt;
    if (szUser) delete [] szUser;
    if (szDomain) delete [] szDomain;
    };
};

// ----------------------------------------------------------------------------

class FilterMessage {
  public:

  FilterRecipient * rHead,   // First recipient name.
                  * rFrom;   // Who the message is from. The reverse path.
  char * szFileName,         // the filename of the message to deliver
       * szSpoolFile,        // the spool filename
       * szMsg,              // In memory buffer for message.
       * szHeloName,         // Hostname given in HELO greeting.
       * szIpName,           // Hostname based on connecting ip address.
       * szIpAddress;        // Connected ip address.
  int iHops,                 // Number of hops found in message headers
      iTid,                  // Thread id.
      iCount;                // Number of recipients.
  u_long ulBytes;            // The size of the message received.

  FilterMessage() {
    rHead = NULL;
    rFrom = NULL;
    szFileName = NULL;
    szSpoolFile = NULL;
    szMsg = NULL;
    szHeloName = NULL;
    szIpName = NULL;
    szIpAddress = NULL;
    iHops = 0;
    iTid = 0;
    iCount = 0;
    ulBytes = 0;
    };  
  ~FilterMessage() {
    if (szFileName) delete [] szFileName;
    if (szSpoolFile) delete [] szSpoolFile;
    if (szMsg) delete [] szMsg;
    if (szHeloName) delete [] szHeloName;
    if (szIpName) delete [] szIpName;
    if (szIpAddress) delete [] szIpAddress;
    };

};

// ----------------------------------------------------------------------------
//
// This class is used by Inet.Mail Perseus to load and store the
// information for a single filter.
//
class Filter {
  public:

  Filter();
  ~Filter();
  int Load();
  int Unload();

  char * szDllName;
  u_long ulEventMask;
  char * szDomain;
  u_long ulPriority;
  int bLoaded;

#ifdef __OS2__

  HMODULE hmod;
  u_long (* pfImMain)(int, u_long *, u_long *, char **);
  u_long (* pfImEvent)(u_long, char *, void *, char **);
  
#elif _WIN32

  HINSTANCE hmod
//  FARPROC pfImMain;
//  FARPROC pfImEvent;

#endif

};

#endif
